% Inspect the integration point quantities.
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
% Input arguments
%     x        - reference geometry field
%     u        - displacement field
%     dT       - temperature difference field
%     output   - this is what you would assign to the output field all of
%                the context argument of the material update() method.
%     component- component of the output array: see the material update()
%                method.
% Output argument
%     fld - the new field that can be used to map values the colors
%
function fld = field_from_integration_points(self, x, u, dT, output, component)
gcells = get(self.feblock,'gcells');
ngcells = length(gcells);
nfens = get(gcells(1),'nfens');
dim = get(x,'dim');
% Material
mat = get(self.feblock, 'mater');
matstates = get(self.feblock, 'matstates');
context.gradU= [];
% Make the inverse map from finite element nodes to geometric cells
f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
gmap=get (f2gmap,'gcell_map');
fenodemap=get (f2gmap,'fenodes');
% Now loop over all gcells in the block
fld =slice(x,1,'fld');
nvals = gather (fld, (1: get(fld,'nfens')),'values');
context.output = output;
for nix=1:length(gmap)
    gl=gmap{nix};
    thisnn=fenodemap{nix};
    if ~isempty(gl) % This node has a element patch in this block
        % establish local numbering of all nodes of the patch @node nix
        lnmap =0*(1:length(gmap));
        for k=1:length(gl)
            kconn = get(gcells(gl(k)), 'conn'); % connectivity
            for j= 1:length(kconn)
                if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
                    lnmap(kconn(j))=kconn(j);
                end
            end
        end
        patchconn=lnmap(lnmap>0);
        % now store the local numbers
        for k=1:length(patchconn)
            lnmap(patchconn(k)) =k;
        end
        lnmap(thisnn) =length(patchconn)+1;
        c = gather(x, thisnn, 'values', 'noreshape'); % coordinates of central node
        % material tangent
        D = tangent_moduli (mat, struct ('ms',matstates{nix,1}));
        es=local_basis(self,[]); % calculate the local Cartesian basis at the node
        % preallocate the strain-displacement matrix
        Nspatialderavg=zeros((length(patchconn)+1),dim);
        Vpatch =0;
        for k=1:length(gl)
            kconn = get(gcells(gl(k)), 'conn'); % connectivity
            lx = gather(x, kconn, 'values', 'noreshape'); % coordinates of nodes
            lx(:,1)=lx(:,1)-c(1);
            lx(:,2)=lx(:,2)-c(2);
            lx(:,3)=lx(:,3)-c(3);
            Nder = Ndermat_param (gcells(gl(k)), [1 1 1]/4);
            xl=lx*es(:,1:3);% local coordinates
            [Nspatialder,detF] = Ndermat_spatial (gcells(gl(k)), Nder, xl);
            Ve = detF * 1/6*1/4;% were taking one force of the tetrahedron
            Vpatch=Vpatch+Ve;
            Nspatialderavg(lnmap(kconn),:)=Nspatialderavg(lnmap(kconn),:)+Ve*Nspatialder;
        end
        Nspatialderavg=Nspatialderavg/ Vpatch;
        conn= [patchconn thisnn];
        U = gather(u, conn, 'values', 'noreshape'); % displacement
        if isempty(dT)
            dTs=zeros(length(conn),1);
        else
            dTs = gather(context.dT, conn, 'values', 'noreshape'); % displacement
        end
        context.gradU = def_grad(gcells(gl(1)), Nspatialderavg, U*es(:,1:3));
        context.dT = dTs(end); % temperature at the node
        [out,ignore] = update(mat, matstates{nix,1}, context);
        nvals (nix) = out(component);
    end
end
fld = scatter(fld, (1: get(fld,'nfens')),nvals);
return;
